from datetime import datetime
from functools import wraps

from flask import flash, url_for
from flask_login import current_user
from werkzeug.utils import redirect

from app import messages
from app.constants import (
    DEADLINE_FOR_STAFF_CHANGES,
    DEADLINE_FOR_STUDENT_CHANGES,
    STAFF,
    STUDENT,
)


def staff_only(func):
    @wraps(func)
    def inner(*args, **kwargs):
        if current_user.role == STAFF:
            return func(*args, **kwargs)
        return redirect(url_for(f"{current_user.role}.projects"))

    return inner


def students_only(func):
    @wraps(func)
    def inner(*args, **kwargs):
        if current_user.role == STUDENT:
            return func(*args, **kwargs)
        return redirect(url_for(f"{current_user.role}.projects"))

    return inner


def only_before_deadline_for_students(func):
    @wraps(func)
    def inner(*args, **kwargs):
        if datetime.utcnow() <= DEADLINE_FOR_STUDENT_CHANGES:
            return func(*args, **kwargs)
        flash(messages.DEADLINE_EXPIRED_FOR_STUDENTS)
        return redirect(url_for(f"{current_user.role}.projects"))

    return inner


def only_before_deadline_for_staff(func):
    @wraps(func)
    def inner(*args, **kwargs):
        if datetime.utcnow() <= DEADLINE_FOR_STAFF_CHANGES:
            return func(*args, **kwargs)
        flash(messages.DEADLINE_EXPIRED_FOR_STAFF)
        return redirect(url_for(f"{current_user.role}.projects"))

    return inner
